@@ -382,6 +382,7 @@ void parseAndPrintFile(char *filename) {
 
 	    f = fopen(filename, "r");
 	    if (f != NULL) {
+		int ret;
 	        int res, size = 3;
 	        char chars[1024];
                 xmlParserCtxtPtr ctxt;
@@ -397,7 +398,12 @@ void parseAndPrintFile(char *filename) {
 		    }
 		    xmlParseChunk(ctxt, chars, 0, 1);
 		    doc = ctxt->myDoc;
+		    ret = ctxt->wellFormed;
 		    xmlFreeParserCtxt(ctxt);
+		    if (!ret) {
+			xmlFreeDoc(doc);
+			doc = NULL;
+		    }
 	        }
 	    }
 	} else if (testIO) {
@@ -474,7 +480,7 @@ void parseAndPrintFile(char *filename) {
 	    if ((fd = open(filename, O_RDONLY)) < 0)
 		return;
 	    base = mmap(NULL, info.st_size, PROT_READ, MAP_SHARED, fd, 0) ;
-	    if (base == MAP_FAILED)
+	    if (base == (void *) MAP_FAILED)
 	        return;
 
 	    doc = xmlParseMemory((char *) base, info.st_size);
@@ -564,7 +570,7 @@ void parseAndPrintFile(char *filename) {
 
 #ifdef LIBXML_DEBUG_ENABLED
     if ((debugent) && (!html))
-	xmlDebugDumpEntities(stdout, doc);
+	xmlDebugDumpEntities(stderr, doc);
 #endif
 
     /*
@@ -582,8 +588,6 @@ int main(int argc, char **argv) {
 #ifdef LIBXML_DEBUG_ENABLED
 	if ((!strcmp(argv[i], "-debug")) || (!strcmp(argv[i], "--debug")))
 	    debug++;
-	else if ((!strcmp(argv[i], "-debugent")) || (!strcmp(argv[i], "--debugent")))
-	    debugent++;
 	else if ((!strcmp(argv[i], "-shell")) ||
 	         (!strcmp(argv[i], "--shell"))) {
 	    shell++;
@@ -644,11 +648,26 @@ int main(int argc, char **argv) {
 	else if ((!strcmp(argv[i], "-nowarning")) ||
 	         (!strcmp(argv[i], "--nowarning"))) {
 	    xmlGetWarningsDefaultValue = 0;
+	    xmlPedanticParserDefault(0);
         }
+	else if ((!strcmp(argv[i], "-pedantic")) ||
+	         (!strcmp(argv[i], "--pedantic"))) {
+	    xmlGetWarningsDefaultValue = 1;
+	    xmlPedanticParserDefault(1);
+        }
+	else if ((!strcmp(argv[i], "-debugent")) ||
+		 (!strcmp(argv[i], "--debugent"))) {
+	    debugent++;
+	    xmlParserDebugEntities = 1;
+	} 
 	else if ((!strcmp(argv[i], "-encode")) ||
 	         (!strcmp(argv[i], "--encode"))) {
 	    i++;
 	    encoding = argv[i];
+	    /*
+	     * OK it's for testing purposes
+	     */
+	    xmlAddEncodingAlias("UTF-8", "DVEnc");
         }
 	else if ((!strcmp(argv[i], "-noblanks")) ||
 	         (!strcmp(argv[i], "--noblanks"))) {
