@@ -9,14 +9,18 @@
 #ifndef __XML_VERSION_H__
 #define __XML_VERSION_H__
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
  * use those to be sure nothing nasty will happen if
  * your library and includes mismatch
  */
 extern void xmlCheckVersion(int version);
-#define LIBXML_VERSION 20202
-#define LIBXML_VERSION_STRING "20202"
-#define LIBXML_TEST_VERSION xmlCheckVersion(20202);
+#define LIBXML_VERSION 20203
+#define LIBXML_VERSION_STRING "20203"
+#define LIBXML_TEST_VERSION xmlCheckVersion(20203);
 
 /*
  * Whether the FTP support is configured in
@@ -45,6 +49,15 @@ extern void xmlCheckVersion(int version);
 #define LIBXML_HTML_DISABLED
 #endif
 
+/*
+ * Whether the Docbook support is configured in
+#if @WITH_SGML@
+#define LIBXML_SGML_ENABLED
+#else
+#define LIBXML_SGML_DISABLED
+#endif
+ */
+
 /*
  * Whether XPath is configured in
  */
@@ -79,6 +92,9 @@ extern void xmlCheckVersion(int version);
 #define DEBUG_MEMORY_LOCATION
 #endif
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
 #endif
 
 
